#!/usr/bin/env bash
#
set -euo pipefail
IFS=$'\n\t'
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

: ${PROFILE:="perf"}

function main {
    local additional_flags=()
    if [ "$PROFILE" = "dev" ] || [ "$PROFILE" = "debug" ]; then
        PROFILE="debug"
    else
        additional_flags+=(--profile "$PROFILE")
    fi
    (cd "$DIR/../" && cargo build --no-default-features "${additional_flags[@]}")
    perf record --freq=2000 --call-graph dwarf --output="$DIR/../perf.data" "$DIR/../target/${PROFILE}/parse" "${@}"

    # Convert to a format firefox will read
    # flags to consider --show-info
    perf script -F +pid --input "$DIR/../perf.data" > "$DIR/../perf.firefox"

    echo "You probably want to go to https://profiler.firefox.com/"
    echo "Either that or run hotspot"
}

main "${@}"
