#!/usr/bin/env bash
#
set -euo pipefail
IFS=$'\n\t'
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

: ${NO_COLOR:=""} # Set to anything to disable color output

REALPATH=$(command -v uu-realpath || command -v realpath)
MAKE=$(command -v gmake || command -v make)

function main {
    local test_names=$(get_test_names "${@}")
    build_container

    local test
    while read test; do
        launch_container "$test"
    done<<<"$test_names"
}

function build_container {
    $MAKE -C "$DIR/../docker/organic_test"
}

function get_test_names {
    local test_file
    local samples_dir=$($REALPATH "$DIR/../org_mode_samples")
    for test_file in "$@"
    do
        if [ -e "$test_file" ]; then
            local test_file_full_path=$($REALPATH "$test_file")
            local relative_to_samples=$($REALPATH --relative-to "$samples_dir" "$test_file_full_path")
            local without_extension="${relative_to_samples%.org}"
            echo "autogen_${without_extension//\//_}" | tr '[:upper:]' '[:lower:]'
        else
            echo "$test_file" | tr '[:upper:]' '[:lower:]'
        fi
    done
}

function launch_container {
    local test="$1"
    local additional_flags=()

    if [ "$NO_COLOR" != "" ]; then
        additional_flags+=(--env "NO_COLOR=$NO_COLOR")
    fi

    local init_script=$(cat <<EOF
set -euo pipefail
IFS=\$'\n\t'

cargo test --no-default-features --features compare --no-fail-fast --lib --test test_loader "$test" -- --show-output
EOF
          )

    docker run "${additional_flags[@]}" --init --rm --read-only --mount type=tmpfs,destination=/tmp -v "$($REALPATH "$DIR/../"):/source:ro" --mount source=cargo-cache,target=/usr/local/cargo/registry --mount source=rust-cache,target=/target --env CARGO_TARGET_DIR=/target -w /source --entrypoint "" organic-test sh -c "$init_script"
}


main "${@}"
